% Copyright (c) 2019, Domenico L. Gatti
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without 
% modification, are permitted provided that the following conditions are 
% met:
% 
%     * Redistributions of source code must retain the above copyright 
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright 
%       notice, this list of conditions and the following disclaimer in 
%       the documentation and/or other materials provided with the 
%       distribution
%       
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
% IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
% PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%
%% Read in images 

header_2 = num2str(img_no);
R = imread([header '/' header_2 '_RFP_' resol '.jpg']);
G = imread([header '/' header_2 '_GFP_' resol '.jpg']);
B = imread([header '/' header_2 '_DP_' resol '.jpg']);
P = imread([header '/' header_2 '_PH_' resol '.jpg']);       
RGB = cat(3,R,G,B);

% Decorrelation stretching
    RGB = decorrstretch(RGB,'Tol',0.01);
    [R,G,B] = imsplit(RGB);

% Linear stretching
    lowhigh = stretchlim(RGB);
    RGB = imadjust(RGB,lowhigh,[0 1],imadj_gamma);
    % If flagged out we use the split from decorrelation stretching
    % [R,G,B] = imsplit(RGB);
    
    RGB_start = figure; imshow(RGB);
            
% Mask regions of the image. 
prompt = 'Load mask of blacked out regions from a file? ''Y''/''N'' [use quotes] [N]: ';
str = input(prompt);
    if isempty(str)||strcmp(str,'N')||strcmp(str,'n')
    elseif strcmp(str,'Y')||strcmp(str,'y')
    load([header '/' header_2 '_nuclei_mask.mat'],'mask_stack');
    mask = any(mask_stack,3);
        R(mask) = 0;
        G(mask) = 0;
        B(mask) = 0;
        P(mask) = 0;
        RGB = cat(3,R,G,B);
        RGB = imadjust(RGB,lowhigh,[0 1],imadj_gamma);
    else
    end

mask_stack = [];
prompt = 'Black out an image region? ''Y''/''N'' [use quotes] [N]: ';
str = input(prompt);
if isempty(str)||strcmp(str,'N')||strcmp(str,'n')
elseif strcmp(str,'Y')||strcmp(str,'y')
    while 1
    prompt = 'Select region? ''Y''/''N'' [use quotes] [Y]: ';
    str = input(prompt);
    if isempty(str)||strcmp(str,'Y')||strcmp(str,'y')
        disp('Use the mouse to select polygon,then choose "Create mask" from the menu')
        mask = roipoly(RGB);        
        R(mask) = 0;
        G(mask) = 0;
        B(mask) = 0;
        P(mask) = 0;
        RGB = cat(3,R,G,B);
        RGB = imadjust(RGB,lowhigh,[0 1],imadj_gamma);
        mask_stack = cat(3,mask_stack,mask);
        if exist([header '/' header_2 '_nuclei_mask.mat'],'file')
            save([header '/' header_2 '_nuclei_mask.mat'],'mask_stack','-append');
        else
            save([header '/' header_2 '_nuclei_mask.mat'],'mask_stack');
        end
    else
        break
    end
    end
else
end

imshow(RGB)

% Crop the image. 
prompt = 'Crop image? ''Y''/''N'' [use quotes] [N]: ';
str = input(prompt);
if isempty(str)||strcmp(str,'N')||strcmp(str,'n')
    
elseif strcmp(str,'Y')||strcmp(str,'y')
    if  isempty(crop_margins)
        prompt = 'Get cropping vector from a file? ''Y''/''N'' [use quotes] [Y]: ';
        str = input(prompt);
        if isempty(str)||strcmp(str,'Y')||strcmp(str,'y')
            load([header '/' header_2 '_nuclei_mask.mat'],'crop_margins');
        else
         
        % Crop images
        close(RGB_start);    
        disp('Choose "Copy Position" from the menu and close the image.')
        J = imcrop(RGB);    
        prompt = 'Save (CTRL-V [Windows] or Cmd-V [Mac]) cropping area as vector:  ';
        crop_margins = input(prompt);
        end
        
        if exist([header '/' header_2 '_nuclei_mask.mat'],'file')
            save([header '/' header_2 '_nuclei_mask.mat'],'crop_margins','-append');
        else
            save([header '/' header_2 '_nuclei_mask.mat'],'crop_margins');
        end
        
        R = imcrop(R,crop_margins);
        G = imcrop(G,crop_margins);
        B = imcrop(B,crop_margins);
        P = imcrop(P,crop_margins);
        
    else    
    R = imcrop(R,crop_margins);
    G = imcrop(G,crop_margins);
    B = imcrop(B,crop_margins);
    P = imcrop(P,crop_margins);
    end
else
end

% Save the starting point for at least one channel as a reference for the
% calculation of the PSNR.
% B_bk = B;

% Background correction.
fprintf('Background correction may introduce artifact. \n')
prompt = 'Carry out background correction? ''Y''/''N'' [use quotes] [N]: ';
str = input(prompt);
if isempty(str)||strcmp(str,'N')||strcmp(str,'n')    
else
    se = strel_bg;
    se_bg = strel('disk',se);
    background = imopen(B,se_bg);
    figure;imshow(background)
    title('Background')
    pause(2)

    R_bc = imtophat(R,strel('disk',se));
    G_bc = imtophat(G,strel('disk',se));
    B_bc = imtophat(B,strel('disk',se));
    P_bc = imtophat(P,strel('disk',se));
    figure;imshowpair(B,B_bc,'montage')
    title('B vs background corrected B')
       
    % Show peak signal to noise ratio (PSNR) improvement
    % [peaksnr, snr] = psnr(im2uint8(B_bc),B_bk);
    % fprintf('PSNR gain is %f4  \n',peaksnr)
    % fprintf('SNR gain is %f4  \n',snr)
    
    prompt = 'Accept background correction? ''Y''/''N'' [use quotes] [N]: ';
    str = input(prompt);
    if isempty(str)||strcmp(str,'N')||strcmp(str,'n') 
    else
        R = R_bc;
        G = G_bc;
        B = B_bc;
        P = P_bc;
    end    
end

    % Save the starting point for at least one channel as a reference for
    % the calculation of the PSNR.
    B_bk = B;

% Denoising using a neural network
% To view the CNN layers execute:
% layers = dnCNNLayers
prompt = 'Carry out denoising using a pretrained convNN (this may take a long time)? ''Y''/''N'' [use quotes] [N]: ';
str = input(prompt);
if isempty(str)||strcmp(str,'N')||strcmp(str,'n')    
else
    net = denoisingNetwork('DnCNN');

    % Denoising only on the luminance channel
    RGB_lab = rgb2lab(RGB);
    [L,a,b] = imsplit(RGB_lab);
    disp('Denoising Luminance')
    L_dn = denoiseImage(L,net);
    RGB_lab_dn = cat(3,L_dn,a,b);
    RGB_dn = lab2rgb(RGB_lab_dn);
    [R,G,B] = imsplit(RGB_dn);
    
    % Denoising each RGB channel
    disp('Denoising R')
    R_dn = denoiseImage(R,net);
    disp('Denoising G')
    G_dn = denoiseImage(G,net);
    disp('Denoising B')
    B_dn = denoiseImage(B,net);
    disp('Denoising P')
    P_dn = denoiseImage(P,net);
    figure;imshowpair(B,B_dn,'montage')
    title('B vs denoised B')

    % Show peak signal to noise ratio (PSNR) improvement
    [peaksnr, snr] = psnr(im2uint8(B_dn),B_bk);
    fprintf('PSNR gain is %f4  \n',peaksnr)
    fprintf('SNR gain is %f4  \n',snr)
    
    prompt = 'Accept denoising? ''Y''/''N'' [use quotes] [Y]: ';
    str = input(prompt);
    if isempty(str)||strcmp(str,'Y')||strcmp(str,'y') 
        R = R_dn;
        G = G_dn;
        B = B_dn;
        P = P_dn;
    else
    end    
end

% Denoising by bilateral filtering
prompt = 'Carry out denoising by bilateral filtering? ''Y''/''N'' [use quotes] [N]: ';
str = input(prompt);
if isempty(str)||strcmp(str,'N')||strcmp(str,'n')    
else
    R_smoothing = var(double(R(:)));
    R_dn = imbilatfilt(R,R_smoothing);
    
    G_smoothing = var(double(G(:)));
    G_dn = imbilatfilt(G,G_smoothing);
    
    B_smoothing = var(double(B(:)));
    B_dn = imbilatfilt(B,B_smoothing);
    
    P_smoothing = var(double(P(:)));
    P_dn = imbilatfilt(P,P_smoothing);

    figure;imshowpair(B,B_dn,'montage')
    title('B vs denoised B from Bilateral Filtering')

    % Show peak signal to noise ratio (PSNR) improvement
    [peaksnr, snr] = psnr(im2uint8(B_dn),B_bk);
    fprintf('PSNR gain is %f4  \n',peaksnr)
    fprintf('SNR gain is %f4  \n',snr)
    
    prompt = 'Accept denoising? ''Y''/''N'' [use quotes] [Y]: ';
    str = input(prompt);
    if isempty(str)||strcmp(str,'Y')||strcmp(str,'y') 
        R = R_dn;
        G = G_dn;
        B = B_dn;
        P = P_dn;
    else
    end  
end

% Make sure all channels are uint8
R = im2uint8(R);
G = im2uint8(G);
B = im2uint8(B);
P = im2uint8(P);

% Carry out denoising by multiscale "context aggregation network" (CAN)
prompt = 'Carry out denoising by multiscale context aggregation network (CAN)? ''Y''/''N'' [use quotes] [N]: ';
str = input(prompt);
if isempty(str)||strcmp(str,'N')||strcmp(str,'n')    
else
    if exist('net')
        clear net
    end
    load('PMI_functions/trainedOperatorLearning-Epoch-181.mat')

    disp('Denoising R')
    IR = cat(3,R,R,R);
    IR_approx = activations(net,IR,'FinalRegressionLayer');
    IR_approx = rescale(IR_approx);
    IR_approx = im2uint8(IR_approx);
    [IR_approx,~,~] = imsplit(IR_approx);
    % imshowpair(R,IR_approx,'montage')

    disp('Denoising G')    
    IG = cat(3,G,G,G);
    IG_approx = activations(net,IG,'FinalRegressionLayer');
    IG_approx = rescale(IG_approx);
    IG_approx = im2uint8(IG_approx);
    [IG_approx,~,~] = imsplit(IG_approx);
    % imshowpair(G,IG_approx,'montage')
    
    disp('Denoising B')
    IB = cat(3,B,B,B);
    IB_approx = activations(net,IB,'FinalRegressionLayer');
    IB_approx = rescale(IB_approx);
    IB_approx = im2uint8(IB_approx);
    [IB_approx,~,~] = imsplit(IB_approx);
    % imshowpair(B,IB_approx,'montage')

    disp('Denoising P')    
    IP = cat(3,P,P,P);
    IP_approx = activations(net,IP,'FinalRegressionLayer');
    IP_approx = rescale(IP_approx);
    IP_approx = im2uint8(IP_approx);
    [IP_approx,~,~] = imsplit(IP_approx);
    % imshowpair(P,IP_approx,'montage')    
    
    
    figure;imshowpair(B,IB_approx,'montage')
    title('B vs denoised B from CAN')

    % Show peak signal to noise ratio (PSNR) improvement
    [peaksnr, snr] = psnr(im2uint8(IB_approx),B_bk);
    fprintf('PSNR gain is %f4  \n',peaksnr)
    fprintf('SNR gain is %f4  \n',snr)
    
    prompt = 'Accept denoising? ''Y''/''N'' [use quotes] [Y]: ';
    str = input(prompt);
    if isempty(str)||strcmp(str,'Y')||strcmp(str,'y') 
        R = IR_approx;
        G = IG_approx;
        B = IB_approx;
        P = IP_approx;
    else
    end  
end

% Make sure all channels are uint8
R = im2uint8(R);
G = im2uint8(G);
B = im2uint8(B);
P = im2uint8(P);

% Apply contrast to each channel and rebuild RGB
    lowhigh = stretchlim(R);
    R = imadjust(R,lowhigh,[0 1],imadj_gamma_R);
    lowhigh = stretchlim(G);
    G = imadjust(G,lowhigh,[0 1],imadj_gamma_G);
    lowhigh = stretchlim(B);
    B = imadjust(B,lowhigh,[0 1],imadj_gamma_B);        
    lowhigh = stretchlim(P);
    P = imadjust(P,lowhigh,[0 1],imadj_gamma_P);        
    RGB = cat(3,R,G,B);


% Grayscale image and its high contrast version
    GS = rgb2gray(RGB);
    GShc = imadjust(GS,lowhigh,[0 1],imadj_gamma);
    
Input_image = figure;
set(gcf, 'Unit','Normalized','Position',[0 0.3 0.7 0.7]);
subplot(2,3,1)
imshow(R)
title('Red Channel')
subplot(2,3,2)
imshow(G)
title('Green Channel')
subplot(2,3,3)
imshow(B)
title('Blue Channel')
subplot(2,3,4)
imshow(P)
title('Phase Contrast Channel')
subplot(2,3,5)
imshow(RGB)
title('Original Image')
subplot(2,3,6)
imshow(GS)
title('Gray Scale Image')


% Channel selection
prompt = 'Remove a channel from the image? ''R''/''G'' [use quotes]: ' ;
del_channel = input(prompt);
if strcmp(del_channel,'R')
    R(:) = 0;
elseif strcmp(del_channel,'G')
    G(:) = 0;
else
end

if strcmp(del_channel,'R')||strcmp(del_channel,'G')        
    RGB = cat(3,R,G,B);
    GS = rgb2gray(RGB);
    figure(Input_image)
    subplot(2,3,1)
    imshow(R)
    title('Red Channel')
    subplot(2,3,2)
    imshow(G)
    title('Green Channel')
    subplot(2,3,3)
    imshow(B)
    title('Blue Channel')
    subplot(2,3,4)
    imshow(P)
    title('Phase Contrast Channel')
    subplot(2,3,5)
    imshow(RGB)
    title('Original Image')
    subplot(2,3,6)
    imshow(GS)
    title('Gray Scale Image')
else
end
    
fprintf('Starting image is defined. \n') 
fprintf('Proceed to identify nuclei? \n') 
fprintf('Type "dbcont" to continue, "dbquit" to exit ! \n') 
keyboard

%% NUCLEI
    switch nucl_method
        case 'adaptive'
        bw = imbinarize(B,'adaptive',"Sensitivity",nucl_sens);

        case 'threshold'
        [level_B,EM_B] = graythresh(B);
        bw = imbinarize(B,threshold_scale_nuclei*level_B);
        
        case 'both'
        bw1 = imbinarize(B,'adaptive',"Sensitivity",nucl_sens);
        [level_B,EM_B] = graythresh(B);
        bw2 = imbinarize(B,threshold_scale_nuclei*level_B);       
        bw = bw1 | bw2;           
    end

    BWs = bwareaopen(bw,area_open_nuclei);

    se90 = strel('line', strel_nuclei, 90); % strelline 3.5
    se45 = strel('line', strel_nuclei, 45); % strelline 3.5
    se0 = strel('line', strel_nuclei, 0);   % strelline 3.5
    BWsdil = imdilate(BWs, [se90 se45 se0]);
    BWdfill = imfill(BWsdil, 'holes');
    % BWnobord = imclearborder(BWdfill, 4);
    BWnobord = BWdfill;
    seD = strel('diamond',1);
    BWfinal = imerode(BWnobord,seD);
    BWfinal = imerode(BWfinal,seD); % 2nd erosion
    BWoutline = bwperim(BWfinal);
    Segout = B;
    Segout(BWoutline) = 255;

    bw_nuclei  = BWfinal;
     
    % After binarization we identify the number of nuclei using an algorithm
    % that looks for disconnected objects.
    cc_nuclei = bwconncomp(bw_nuclei,8);
    n_nuclei = cc_nuclei.NumObjects;

    % Check the nuclei area
    S_nuclei = regionprops(cc_nuclei,'Area','Centroid');
    mean_area = mean([S_nuclei.Area]);
    std_area = std([S_nuclei.Area]);

    % Plot masks and labels to identify nuclei
    nuclei_labeled_mat = labelmatrix(cc_nuclei);
    Label_matrix = figure;
    set(gcf,'Unit','Normalized','Position',[0. 0.2 0.3 0.5])
    imagesc(nuclei_labeled_mat);hold on
%     for i = 1:n_nuclei
%         text(S_nuclei(i).Centroid(1)-15,S_nuclei(i).Centroid(2),num2str(i));
%     end
    hold off;title('Nuclei labels')
    pause(2)
    
    RGB_marked = figure;
    imshowpair(RGB,Segout,'montage');hold on
    for i = 1:n_nuclei
        text(S_nuclei(i).Centroid(1)-15+size(B,2),S_nuclei(i).Centroid(2),...
            num2str(i),'Color','red','FontSize',14);
    end
    hold off; title('RGB versus marked nuclei')
    
    
% REMOVE PARTIAL NUCLEI OR ARTIFACTS

% Some nuclei are at the edge of the image; we can easily identify them, and, 
% perhaps remove them from the analysis.
small_nuclei_ind = find([S_nuclei.Area] < mean_area - std_scale_small.*std_area);
small_nuclei = false(size(bw_nuclei));

% Large nuclei are probably artifacts and should be removed from all images
% Including cell, cytoplasm, and mitochondria
large_nuclei_ind = find([S_nuclei.Area] > mean_area + std_scale_large*std_area);
large_nuclei = false(size(bw_nuclei));

% Identify small nuclei
small_nuclei_ind
prompt = 'Enter a modified small nuclei list (as vector; [] for no change, [0] to empty the list):  ';
    small_nuclei_change = input(prompt);
    if ~isempty(small_nuclei_change)
    small_nuclei_ind = small_nuclei_change;
    end
    if small_nuclei_ind == 0
        small_nuclei_ind = [];
    end

if exist([header '/' header_2 '_nuclei_mask.mat'],'file')
    prompt = 'Load additional small nuclei to delete from file? ''Y''/''N'' [use quotes] [Y] : ';
        str = input(prompt);
        if isempty(str) || strcmp(str,'Y') || strcmp(str,'y')
            s1 = load([header '/' header_2 '_nuclei_mask.mat'],'del_small','small_nuclei_ind');
            small_nuclei_ind = cat(2,small_nuclei_ind,unique(s1.small_nuclei_ind));
            small_nuclei_ind = unique(cat(2,small_nuclei_ind, s1.del_small))
        end
end
    
prompt = 'Enter additional small nuclei to delete (as vector; [] for no deletion):  ';
    del_small = input(prompt);
    if ~isempty(del_small)
    small_nuclei_ind = cat(2,small_nuclei_ind,del_small)
    elseif ~isnumeric(del_small)    
    end

    if exist('s1.del_small','var')
        del_small = union(s1.del_small,del_small);
    end
       

    for i = 1:length(small_nuclei_ind)
    small_nuclei(cc_nuclei.PixelIdxList{small_nuclei_ind(i)}) = true;
    end

    if exist([header '/' header_2 '_nuclei_mask.mat'],'file')
        save([header '/' header_2 '_nuclei_mask.mat'],'del_small','small_nuclei_ind','-append');
    else
        save([header '/' header_2 '_nuclei_mask.mat'],'del_small','small_nuclei_ind');
    end
        

% Identify large nuclei.
large_nuclei_ind
prompt = 'Enter a modified large nuclei list (as vector; [] for no change, [0] to empty the list):  ';
    large_nuclei_change = input(prompt);
    if ~isempty(large_nuclei_change)
    large_nuclei_ind = large_nuclei_change;
    end
    if large_nuclei_ind == 0
        large_nuclei_ind = [];
    end


prompt = 'Load additional large nuclei to delete from file? ''Y''/''N'' [use quotes] [N]: ';
    str = input(prompt);
    if isempty(str) || strcmp(str,'N') || strcmp(str,'n')
    elseif strcmp(str,'Y') || strcmp(str,'y')
        try 
            s2 = load([header '/' header_2 '_nuclei_mask.mat'],'del_large','large_nuclei_ind');
        catch
        end
        if exist('s2.del_large','var')||exist('s2.large_nuclei_ind','var')    
        % s2 = load([header '/' header_2 '_nuclei_mask.mat'],'del_large','large_nuclei_ind');
        large_nuclei_ind = cat(2,large_nuclei_ind,s2.large_nuclei_ind);
        large_nuclei_ind = cat(2,large_nuclei_ind,s2.del_large)
        end
    else
    end

prompt = 'Enter additional large nuclei to delete (as vector; [] for no deletion):  ';
    del_large = input(prompt);
    if ~isempty(del_large)
    large_nuclei_ind = cat(2,large_nuclei_ind,del_large)
    elseif ~isnumeric(del_large)
    end
    
    if exist('s2.del_large','var')
        del_large = union(s2.del_large,del_large);
    end
    
    for i = 1:length(large_nuclei_ind)
    large_nuclei(cc_nuclei.PixelIdxList{large_nuclei_ind(i)}) = true;
    end

    if exist([header '/' header_2 '_nuclei_mask.mat'],'file')
        save([header '/' header_2 '_nuclei_mask.mat'],'del_large','large_nuclei_ind','-append');
    else
        save([header '/' header_2 '_nuclei_mask.mat'],'del_large','large_nuclei_ind','-append');
    end
                
% Remove small and large nuclei
bw_nuclei = bw_nuclei & ~small_nuclei;
bw_nuclei = bw_nuclei & ~large_nuclei;

% Add nuclei manually
nmask = [];

    prompt = 'Load additional nuclei masks from file? ''Y''/''N'' [use quotes] [Y]: ';
    str = input(prompt);
    
    if isempty(str)||strcmp(str,'Y')||strcmp(str,'y')
        load([header '/' header_2 '_nuclei_mask.mat'],'nmask')
        nuclei_mask = figure;
        h_im = imshow(RGB);

        for i = 1:size(nmask,1)
            e = drawellipse('Center',nmask(i,1:2),'SemiAxes',nmask(i,3:4),'RotationAngle',nmask(i,5),'StripeColor','r');
            BW = createMask(e,h_im);
            bw_nuclei = bw_nuclei | BW;
        end              
    else
    end


    % Update nuclei count and masks.
    cc_nuclei = bwconncomp(bw_nuclei,8);
    n_nuclei = cc_nuclei.NumObjects;

    nuclei_labeled_mat = labelmatrix(cc_nuclei);
    figure(Label_matrix);
    set(gcf,'Unit','Normalized','Position',[0. 0.2 0.3 0.5])
    imagesc(nuclei_labeled_mat);
    title('Nuclei labels')
    
    BWoutline = bwperim(bw_nuclei);
    
    Segout = B;
    Segout(BWoutline) = 255;
    
    % Nuclear boundaries prior to additional changes in the mask
    [nuclei_boundaries,nuclei_L,nuclei_n,nuclei_A] = bwboundaries(bw_nuclei,'noholes');    
    Contoured_nuclei_1 = figure;
    imshow(B)
        hold on
        for k = 1:length(nuclei_boundaries)
           boundary = nuclei_boundaries{k};
           plot(boundary(:,2), boundary(:,1), 'r', 'LineWidth', 4)
        end
    
    close(RGB_marked);
    RGB_marked = figure;
    imshowpair(RGB,Segout,'montage')
    title('RGB versus marked nuclei')

    while 1
        prompt = 'Add more nuclei manually? ''Y''/''N'' [use quotes] [Y]: ';
        str = input(prompt);
        if isempty(str)||strcmp(str,'Y')||strcmp(str,'y')
            disp('Use the mouse to shape the ellipse.')
            e = drawellipse('Center',[100,100],'SemiAxes',[50,50],'StripeColor','r');
            disp('Press RETURN to create the mask')
            pause
            try
            e_center = e.Center;
            catch
            end
            if ~exist('e_center','var')
                break
            else
                BW = createMask(e,RGB);
                bw_nuclei = bw_nuclei | BW;
                nmask_current = [e.Center e.SemiAxes e.RotationAngle e.AspectRatio];
                nmask = cat(1,nmask,nmask_current);
            end
        else
            if ~isempty(nmask)
            close(gcf)
            save([header '/' header_2 '_nuclei_mask.mat'],'nmask','-append');
            end
            break
        end
    end


% Load a polygonal mask to fill nuclear areas
    BW_nuclei_1 = figure;imshow(bw_nuclei)

    prompt = 'Load polygonal mask to fill nuclei from file? ''Y''/''N'' [use quotes] [Y]: ';
    str = input(prompt);
    
    if isempty(str)||strcmp(str,'Y')||strcmp(str,'y')
        try             
        load([header '/' header_2 '_nuclei_mask.mat'],'poly_mask')
        catch
        end
        if exist('poly_mask','var')
            poly_mask = logical(poly_mask);
            bw_nuclei(poly_mask) = 1;
        else
            poly_mask = false(size(bw_nuclei));
        end
    else
        poly_mask = false(size(bw_nuclei));        
    end
    
    % imshow(bw_nuclei)
    imshow(Segout)

% Draw a poligonal area
    while 1
        prompt = 'Fill nuclei with a polygonal area? ''Y''/''N'' [use quotes] [Y]: ';
        str = input(prompt);
        if isempty(str)||strcmp(str,'Y')||strcmp(str,'y')
            disp('Use the mouse to draw a polygon.')
            e = drawpolygon('LineWidth',3,'Color','cyan');
            disp('Press RETURN to create the mask')
            pause
            try
            e_position = e.Position;
            catch
            end            
            if ~exist('e_position','var')
                break
            else
                BW = createMask(e,RGB);
                poly_mask = poly_mask | BW;               
            end
        else
            close(gcf)
            save([header '/' header_2 '_nuclei_mask.mat'],'poly_mask','-append');
            break
        end
    end

% Add polygons to nuclei
if ~isempty(find(poly_mask, 1))
    bw_nuclei(poly_mask) = 1;
else
end

% Final fill
bw_nuclei = imfill(bw_nuclei, 'holes');

% IMPORTANT: before adding a separation mask we backup the nuclei mask so
% that the total nuclei area and various ratios can be calculated
% correctly.
bw_nuclei_uncut = bw_nuclei;

% Load a separating mask for clustered nuclei
    BWoutline = bwperim(bw_nuclei);
    Segout = B;
    Segout(BWoutline) = 255;

    BW_nuclei = figure;imshow(Segout)

    prompt = 'Load separating mask for clustered nuclei from file? ''Y''/''N'' [use quotes] [Y]: ';
    str = input(prompt);
    
    if isempty(str)||strcmp(str,'Y')||strcmp(str,'y')
        try             
        load([header '/' header_2 '_nuclei_mask.mat'],'separ_mask')
        catch
        end
        if exist('separ_mask','var')
        bw_nuclei(logical(separ_mask)) = 0; 
        else
        separ_mask = zeros(size(bw_nuclei));    
        end
    else
        separ_mask = zeros(size(bw_nuclei));
    end
    
    imshow(Segout)
    
    while 1
        prompt = 'Separate clustered nuclei manually? ''Y''/''N'' [use quotes] [Y]: ';
        str = input(prompt);
        if isempty(str)||strcmp(str,'Y')||strcmp(str,'y')
            disp('Use the mouse to draw a separating line.')
            e = drawline('SelectedColor','yellow');
            disp('Press RETURN to create the mask')
            pause
            try
            e_position = e.Position;
            catch 
            end                        
            if ~exist('e_position','var')
                break
            else
                BW = createMask(e,RGB);
                se90 = strel('line', strel_separ, 90);
                se45 = strel('line', strel_separ, 45);
                se0 = strel('line', strel_separ, 0);
                BW = imdilate(BW, [se90 se45 se0]);
                % BW = imdilate(BW, [se90 se45 se0]); % 2nd dilation
                separ_mask = separ_mask | BW;               
            end
        else
            close(gcf)
            save([header '/' header_2 '_nuclei_mask.mat'],'separ_mask','-append');
            break
        end
    end

% Split clustered nuclei
if ~isempty(find(separ_mask, 1))
    bw_nuclei(logical(separ_mask)) = 0;
else
end

% Final dilation
%     se90 = strel('line', strel_nuclei_2, 90); % strelline 3.5
%     se45 = strel('line', strel_nuclei_2, 45); % strelline 3.5
%     se0 = strel('line', strel_nuclei_2, 0);   % strelline 3.5
%     bw_nuclei = imdilate(bw_nuclei, [se90 se45 se0]);
% 
% 
    BWoutline = bwperim(bw_nuclei);
    Segout = B;
    Segout(BWoutline) = 255;

figure;imshowpair(RGB, bw_nuclei,'montage')
title('RGB versus final nuclei')

figure;imshow(Segout)
title('Final nuclei with perimeter')

% Update nuclei boundaries
    [nuclei_boundaries,nuclei_L,nuclei_n,nuclei_A] = bwboundaries(bw_nuclei,'noholes');    
    Contoured_nuclei_2 = figure;
    imshow(B)
        hold on
        for k = 1:length(nuclei_boundaries)
           boundary = nuclei_boundaries{k};
           plot(boundary(:,2), boundary(:,1), 'r', 'LineWidth', 4)
        end

% Update nuclei count and masks.
cc_nuclei = bwconncomp(bw_nuclei,8);
n_nuclei = cc_nuclei.NumObjects;
S_nuclei = regionprops(cc_nuclei,'Centroid');

nuclei_labeled_mat = labelmatrix(cc_nuclei);
figure(Label_matrix);
set(gcf,'Unit','Normalized','Position',[0. 0.2 0.3 0.5])
imagesc(nuclei_labeled_mat);hold on
for i = 1:n_nuclei
    text(S_nuclei(i).Centroid(1)-15,S_nuclei(i).Centroid(2),num2str(i));
end
hold off;
title('Nuclei labels')

% disp('Press any key to continue !') 
% pause;

%%
fprintf('Nuclei mask is complete. Identify cell boundaries?  \n')
fprintf('Type "dbcont" to continue, "dbquit" to exit! \n') 
keyboard
 
%% CELL and CYTOPLASM
prompt = 'Select channel to use for adaptive cell boundaries; ''R''/''G''/''GS'' [use quotes] [GS]: ';
adapt_ch = input(prompt);
if isempty(adapt_ch)
    adapt_ch = eval('GS');
else
    adapt_ch = eval(input(prompt));    
end

% Increase contrast
% lowhigh = stretchlim(adapt_ch);
% adapt_ch = imadjust(adapt_ch,lowhigh,[0 1],imadj_gamma_2);

switch cell_method
    case 'adaptive'
    % sens = 0.6;
    bw = imbinarize(adapt_ch,'adaptive',"Sensitivity",cell_sens);
    
    case 'threshold'
    % Here we binarize the cell and the cytoplasm using a different scale
    [level_ac,EM_ac] = graythresh(adapt_ch);
    bw = imbinarize(adapt_ch,threshold_scale_cells*level_ac); 
    
    case 'both'
    bw1 = imbinarize(adapt_ch,'adaptive',"Sensitivity",cell_sens);
    [level_ac,EM_ac] = graythresh(adapt_ch);
    bw2 = imbinarize(adapt_ch,threshold_scale_cells*level_ac); 
    bw = bw1 | bw2;               
end

    BWs = bwareaopen(bw,area_open_cell);

    se90 = strel('line', strel_cell, 90);
    se45 = strel('line', strel_cell, 45);
    se0 = strel('line', strel_cell, 0);
    BWsdil = imdilate(BWs, [se90 se45 se0]);

    prompt = 'Fill holes in the cell mask? ''Y''/''N'' [N] [use quotes]: ';
    fill = input(prompt);
    if isempty(fill)||strcmp(fill,'N')||strcmp(fill,'n')
        BWdfill = BWsdil;
    else
        BWdfill = imfill(BWsdil, 'holes');
    end

  % BWnobord = imclearborder(BWdfill, 4);
    BWnobord = BWdfill;
    seD = strel('diamond',1);
    BWfinal = imerode(BWnobord,seD);
    % BWfinal = imerode(BWfinal,seD);
    BWoutline = bwperim(BWfinal);
    % Segout = adapt_channel;
    Segout = adapt_ch;
    Segout(BWoutline) = 255;

    % Add nuclei outside cell boundaries 
    se90 = strel('line', strel_cell_nuclei, 90);
    se45 = strel('line', strel_cell_nuclei, 45);
    se0 = strel('line', strel_cell_nuclei, 0);
    
    figure;imshow(Segout), title('outlined cells');
    
    prompt = 'Add nuclei outside initial cell boundaries? ''Y''/''N'' [use quotes] [Y]: ';
    str = input(prompt);
    if isempty(str)||strcmp(str,'Y')||strcmp(str,'y')
        dilated_bw_nuclei = imdilate(bw_nuclei, [se90 se45 se0]);
        BWfinal(dilated_bw_nuclei) = 1;
        % BWfinal = imfill(BWfinal, 'holes');
    else
    end

    figure, imshowpair(Segout,BWfinal,'montage'), title('Outlined cells versus cell mask');
    bw_cell = BWfinal;
    
    
    % Display cell boundaries
    pause(2)
    [cell_boundaries,cell_L,cell_n,cell_A] = bwboundaries(bw_cell);
    Contoured_cells = figure;
    imshow(adapt_ch, [])
        hold on
        for k = 1:length(cell_boundaries)
           cell_boundary = cell_boundaries{k};
           plot(cell_boundary(:,2), cell_boundary(:,1), 'r', 'LineWidth', 4)
        end
    

%     disp('Press any key to continue !') 
%     pause;

% bw_cell = bw_cell & ~large_nuclei;

% Remove nuclei outside the cell boundaries
prompt = 'Remove nuclei outside cell boundaries? ''Y''/''N'' [use quotes] [N]: ';
str = input(prompt);
if isempty(str)||strcmp(str,'N')||strcmp(str,'n')
    else
    bw_nuclei = bw_nuclei & bw_cell;
end

bw_cyto = bw_cell & ~bw_nuclei;

figure;
imshowpair(RGB,bw_cyto,'montage')
title('RGB versus cytoplasm')

% disp('Press any key to continue !') 
% pause;

%%
fprintf('Cell outline is complete! \n')
fprintf('Identify other features of importance? \n') 
fprintf('Type "dbcont" to continue, "dbquit" to exit! \n') 
keyboard

%% OTHER FEATURE
% Here we binarize the OTHER feature of interest by selecting a channel to threshold.
prompt = 'Which channel contains the other important feature of the image? R/G [do not use quotes]: ';
str = input(prompt);
if ischar(str)
    OF = eval(str);
else
    OF = str;
end
[level_OF,EM_OF] = graythresh(OF);
bw = imbinarize(OF,threshold_scale_other*level_OF);
bw_other = bwareaopen(bw,area_open_other);
bw_other(bw_nuclei) = 0;
bw_other(~bw_cell) = 0;
bw_other(~bw_cyto) = 0;

OF(~bw_other)= 0;

figure;
imshowpair(RGB,OF,'montage')
title('RGB versus masked OTHER')

% disp('Press any key to continue !') 
% pause;

%%
fprintf('OTHER feature selection is complete! \n') 
fprintf('Do you want to esclude some image regions? \n') 
fprintf('Type "dbcont" to continue, "dbquit" to exit! \n') 
keyboard

%% FINAL CLEANUP
prompt = 'Load mask of blacked out regions from a file? ''Y''/''N'' [use quotes] [Y]: ';
str = input(prompt);
    if isempty(str)||strcmp(str,'Y')||strcmp(str,'y')
    load([header '/' header_2 '_nuclei_mask.mat'],'mask_stack');
    mask = any(mask_stack,3);
        R(mask) = 0;
        G(mask) = 0;
        B(mask) = 0;
        OF(mask) = 0;
        bw_nuclei(mask) = 0;
        bw_nuclei_uncut(mask) = 0;
        bw_cell(mask) = 0; 
        bw_cyto(mask) = 0;
        bw_other(mask) = 0;
        RGB = cat(3,R,G,B);
    else
    end

if ~exist('mask','var') || isempty(mask)
    mask = zeros(size(B));
    mask_stack = [];
else
    mask_stack = mask;
end

figure;
imshowpair(RGB,mask,'montage');title('Update image with mask of blacked out region')

while 1
    prompt = 'Black out an image region ? ''Y''/''N'' [Y]: ';
    str = input(prompt);
    if isempty(str)||strcmp(str,'Y')
        disp('Use the mouse to select polygon,then choose "Create mask" from the menu')
        mask = roipoly(RGB);        
        R(mask) = 0;
        G(mask) = 0;
        B(mask) = 0;
        OF(mask) = 0;
        GS(mask) = 0;
        bw_nuclei(mask) = 0;
        bw_nuclei_uncut(mask) = 0;
        bw_cell(mask) = 0; 
        bw_cyto(mask) = 0;
        bw_other(mask) = 0;
        RGB = cat(3,R,G,B);
        mask_stack = cat(3,mask_stack,mask);
    else
        break
    end
end

save([header '/' header_2 '_nuclei_mask.mat'],'mask_stack','-append');

Final_img = figure;
imshow(RGB);

% Since the nuclei mask is finalized here we extract the properties of each nucleus:
% 
% Maj_AL: major axis length
% 
% Min_AL: mino axis length
% 
% Peri: perimeter of a nucleus in number of pixels
% 
% Circ:  circularity of a nucleus (Circularity specifies the roundness of objects. 
% For a perfect circle, the circularity value is 1. )
% 
% Eccentr: eccentricity of a nucleus (The eccentricity is the ratio of the distance 
% between the foci of the ellipse and its major axis length. The value is between 
% 0 and 1. An ellipse whose eccentricity is 0 is actually a circle, while an ellipse 
% whose eccentricity is 1 is a line segment.
% 
% Area: number of pixels of each nucleus
% 
% Conv_area: number of pixels in the binary mask identifying the nucleus. Slightly 
% larger than area.
% 
% Centroids: coordinates of the nuclei centroids.
%
% MeanIntensity: mean intensity of the pixels of the corresponding
% grayscale image for each object
% 
% Mostly we are interested in the means of these properties:

    cc_nuclei = bwconncomp(bw_nuclei,8);
    n_nuclei = cc_nuclei.NumObjects;

    S_nuclei = regionprops(cc_nuclei,B,'All');

    Maj_AL = mean([S_nuclei.MajorAxisLength]);
    Min_AL = mean([S_nuclei.MinorAxisLength]);
    Peri = mean([S_nuclei.Perimeter]);
    Circ = mean([S_nuclei.Circularity]);
    Eccentr = mean([S_nuclei.Eccentricity]);
    Area = mean([S_nuclei.Area]);
    Conv_area = mean([S_nuclei.ConvexArea]);
    Centroids = reshape([S_nuclei.Centroid],2,n_nuclei)';
    Radius = mean([S_nuclei.EquivDiameter])/2;
    Mean_int = [S_nuclei.MeanIntensity];

    % Large nuclei
    large_nuclei_ind = find([S_nuclei.Area] > Area + std_scale_large*std([S_nuclei.Area]));
    large_nuclei = false(size(bw_nuclei));
    
    % Centroids distances
    centr_dist_mat = zeros(n_nuclei,n_nuclei);
    for i = 1:n_nuclei
        for j = i:n_nuclei
            centr_dist_mat(i,j) = norm(Centroids(i,:) - Centroids(j,:));
        end
    end
    
    nz_ind = centr_dist_mat > 0;
    
    % Mean distance between every nucleus and every other nucleus.
    mean_centr_dist = mean(centr_dist_mat(nz_ind))*pix2micr;
    std_centr_dist = std(centr_dist_mat(nz_ind))*pix2micr;
    
    % Mean distance between every nucleus and every other nucleus (by nucleus).
    centr_dist_mat_all = centr_dist_mat + centr_dist_mat';
    mean_centr_dist_by_nucl = sum(centr_dist_mat_all,2)*pix2micr/(n_nuclei-1);
    std_centr_dist_by_nucl = std(centr_dist_mat_all,0,2);

    % Nuclei axes ratio
    AL_ratio = Maj_AL/Min_AL;

    % Label matrix
    close(Label_matrix)
    nuclei_labeled_mat = labelmatrix(cc_nuclei);
    Label_matrix = figure;
    set(gcf,'Unit','Normalized','Position',[0. 0.2 0.3 0.8])
    subplot(2,1,1);imagesc(nuclei_labeled_mat);colorbar;hold on
    for i = 1:n_nuclei
        text(S_nuclei(i).Centroid(1)-15,S_nuclei(i).Centroid(2),num2str(i));
    end
    hold off;title('Nuclei labels')
    pause(2)

    title('Nuclei labels')

    % Nuclei area distribution
    subplot(2,1,2)
    histogram([S_nuclei.Area]); 
    mean_area = mean([S_nuclei.Area]);
    std_area = std([S_nuclei.Area]);
    title('Histogram of Nuclei Area')
    ylim([0 10])
    
    % Nuclei/cell ratio
    nucl_cell_ratio = sum(bw_nuclei(:))/sum(bw_cell(:));
    uncut_nucl_cell_ratio = sum(bw_nuclei_uncut(:))/sum(bw_cell(:));

    % Nuclei/cytoplasm ratio
    nucl_cyto_ratio = sum(bw_nuclei(:))/sum(bw_cyto(:));
    uncut_nucl_cyto_ratio = sum(bw_nuclei_uncut(:))/sum(bw_cyto(:));

    % Other ratios
    other_cyto_ratio = sum(bw_other(:))/sum(bw_cyto(:));
    other_cell_ratio = sum(bw_other(:))/sum(bw_cell(:));
    other_nuclei_ratio = sum(bw_other(:))/sum(bw_nuclei(:));
    
% end

% disp('Press any key to continue !') 
% pause;

fprintf('All cell boundaries are complete! \n')
fprintf('Proceed to calculate distances between nuclei and other feature? \n')
fprintf('Type "dbcont" to continue, "dbquit" to exit ! \n') 
keyboard

%% DISTANCES BY NUCLEUS 
% Here we want to determine the profiles of the distances of the OTHER
% feature from nuclei. In standard applications of image analysis once we
% move an image in frequency space by using the FFT we can apply there a
% filter and come back into standard space by applying an inverse FFT
% (iFFT). One property of the FFT is that if we calculate the iFFT removing
% the phases and using only the squared amplitudes, we obtain in standard
% space a new type of map (called a Patterson map), which contains all the
% vectors from every pixel to every other pixel in the image. We calculate
% a Patterson map separately for each nucleus.
%  
% Thus, we can calculate:
% 
% 1. a Patterson map of the image of nuclei + other feature
% 
% 2. a Patterson map of the image of nuclei 
% 
% 3. a Patterson map of the image of other feature
% 
% Then, we obtain all the vectors from the other feature to the nuclei 
% by subtracting 2+3 from 1.
%
% Likewise, to calculate the distance of each nucleus from every other
% nucleus we subtract from the Patterson map of all nuclei (vectors between
% all nuclei) the sum of the Patterson map of an individual nucleus (all vectors
% inside that nucleus) and the Patterson map of all the nuclei except that
% nucleus.
 
% Here we prepare a matrix that for each pixel contains the distance of
% that pixel from the image center,
[rows,cols] = size(bw_nuclei);

% IMPORTANT: no need to use the original dimensions because the fft is a
% cyclic, infinite function and pixel dimensions are retained even if the
% size of the fft and ifft changes.
% m = rows;
% n = cols;

% FFT dimensions (as a power of 2 for speed and accuracy).
m = 2^nextpow2(rows);
n = 2^nextpow2(cols);

% Distance of every pixel from the image center (we will use this later on)
% dist_mat = zeros(m,n);
[xx,yy] = meshgrid(-m/2:m/2-1,-n/2:n/2-1); 
dist_mat = sqrt(xx.^2+yy.^2)'; % Notice the transpose !

% Take into account pixel size (0.16 microns at 40x). Better done later in
% order to generate a finer grid of distances.
% dist_mat = pix2micr*sqrt(xx.^2+yy.^2); 

% Image matrix with every element replaced with the pixel distance from the
% center
dist_mat = ceil(dist_mat);
dist_unique = unique(dist_mat);
nunique = length(dist_unique);


% k stores for every unique distance all the indices of the image pixels
% that correspond to that distance.
k = cell(nunique,1);
for j = 1:nunique
    r = dist_unique(j);
    k{j} = find(dist_mat == r);
end

%% RADIAL PROFILES
% Here we set up a loop that will go over each nucleus doing an FFT and an
% iFFT bringing back an image containing all the vectors from the other
% feature to the nucleus and an image containing all the vectors from each
% nucleus to every other nucleus. After calculating each Patterson map we
% calculate a radial average of the vector densities which provide a
% profile of the distribution of distances of the other feature from each
% nucleus.

% Empty cell to store the individual nuclei profiles with respect to the
% other feature (OF).
Radial_Profiles = cell(n_nuclei,1);

% Empty cell to store the individual nuclei profiles with respect to every
% other nucleus.
Radial_Profiles_n = cell(n_nuclei,1);

% FFT of all the nuclei.
F_n_a = fft2(B,m,n); 

% Intensities. But this is not needed as shown below.
% I_n_a = real(F_n_a).^2;

% FFT of all the other feature.
OF_masked = OF;
OF_masked(~bw_other)= 0;

F_m = fft2(OF_masked,m,n); 

% Intensities.
I_m = real(F_m).^2;

% Loop over all nuclei
for i = 1:n_nuclei
    
nucleus_no = i;
other_nuclei = true(size(bw_nuclei));
other_nuclei(cc_nuclei.PixelIdxList{nucleus_no}) = false;
bw_centroid = B;

% Here we zero any nucleus different from the one under consideration.
bw_centroid(other_nuclei) = 0;

% FFT of each nucleus
F_n_i = fft2(bw_centroid,m,n);

% NUCLEUS TO OTHER FEATURE DISTANCE ---------------------------------------

% FFt of each nucleus + other feature. The FFT is a linear function ,
% therefore no need to calculate F_nm separately.
F_nm = F_n_i + F_m;

% Patterson map. We carry out the subtraction in frequency space doing a
% single iFFT.
p_d0 = abs(ifft2(real(F_nm).^2 - real(F_n_i).^2 - I_m,'symmetric'));

% Here we shift the F(0,0) of the fft to the center of the transform.
p_d0 = fftshift(p_d0);

% NUCLEUS TO NUCLEUS DISTANCE ---------------------------------------------

% FFT of all nuclei except the nucleus under consideration. The FFT is a
% linear function , therefore no need to calculate F_n_ai separately. But
% also this is not needed as shown below.
% F_n_ai = F_n_a - F_n_i;

% Patterson map. We carry out the subtraction in frequency space doing a
% single iFFT.
% p_d0_n = abs(ifft2(-real(F_n_i).^2 - real(F_n_ai).^2 + I_n_a,'symmetric'));

% This is the same as:
% p_d0_n = abs( ifft2(2*real(F_n_a).*real(F_n_i) - 2*real(F_n_i).^2 ,'symmetric'));

% Simplifying further:
p_d0_n = abs( ifft2(2*(real(F_n_a)-real(F_n_i)).*real(F_n_i) ,'symmetric'));

% Here we shift the F(0,0) of the fft to the center of the transform.
p_d0_n = fftshift(p_d0_n);

% RADIAL PROFILES ---------------------------------------------------------

% Radial profiles of the distance of the other feature with respect to each
% nucleus.
radial_profile_p_d0 = zeros(nunique,2);

% Radial profiles of the distance between each nucleus and every other
% nucleus.
radial_profile_p_d0_n = zeros(nunique,2);

for j = 1:nunique
    radial_profile_p_d0(j,1) = mean(p_d0(k{j}));
    radial_profile_p_d0_n(j,1) = mean(p_d0_n(k{j}));
end

% Here we store the profiles for each nucleus for later plotting. Each
% profile is scaled by the (area of corresponding nucleus*mean pixel
% brightness of the nucleus)
Radial_Profiles{i} = radial_profile_p_d0/(S_nuclei(i).Area * S_nuclei(i).MeanIntensity);
Radial_Profiles_n{i} = radial_profile_p_d0_n/(S_nuclei(i).Area * S_nuclei(i).MeanIntensity);

end

%% NUCLEUS TO OTHER FEATURE DISTANCE
% Mean and std of the peak positions
dist_cutoff_ind = uint64((dist_cutoff/pix2micr)+1);
mdist = zeros(n_nuclei,1);
mdens = zeros(n_nuclei,1);
for i = 1:n_nuclei
    % [pd,pd_ind] = findpeaks(smooth(Radial_Profiles{i}(1:dist_cutoff_ind,1),5));
    [pd,pd_ind] = findpeaks(smooth(Radial_Profiles{i}(1:dist_cutoff_ind,1),'rlowess'),'SortStr','descend','NPeaks',3);

    if ~isempty(pd)
        mdens(i) = pd(1);
        mdist(i) = dist_unique(pd_ind(1))*pix2micr;
    else
        mdens(i) = NaN;
        mdist(i) = NaN;
    end
end
mean_mdist = nanmean(mdist);
std_mdist = nanstd(mdist);
mean_mdens = nanmean(mdens);
std_mdens = nanstd(mdens);
 
% The Patterson map is centrosymmetric. The center of the image is the
% F(0,0) vector (the vector of every pixel with itself). Since this is
% properly a "Difference Patterson Map" (1 - 2+3, as above) that point has
% 0 density. Hot regions are regions with high density of vectors. Next we
% calculate a radially averaged density for each Patteron map, which gives
% us the vector distribution.
mnr = Radius*pix2micr;

if n_nuclei <= 100
    Other_Feature_Profiles_plot = figure;
    set(gcf,'Unit','Normalized','Position',[0 0.1 0.6 0.9]);
    for i = 1:n_nuclei
        subplot(6,ceil(n_nuclei/6),i)
        plot(dist_unique*pix2micr,Radial_Profiles{i}(:,1), 'b-', 'LineWidth', 2);
        hold on
        ax = gca;
        line([dist_cutoff dist_cutoff],ax.YLim,'Color','magenta','LineStyle','-')
        line([mnr mnr],ax.YLim,'Color','green','LineStyle','-')
        grid on;
        title(['N' num2str(i)], 'FontSize', 12)
    if n_nuclei<30
        xlabel('Centr. Dist. [\mu]')
    end
        % ylabel('Vector density')
        xlim([0 100])
        xticks([0 20 40 60 80 100]);
        xticklabels([]);
    end
else
    fprintf('Too many nuclei to display profiles. \n')
end

% Recommend which nuclei should be deleted
std_nuclear_area = std([S_nuclei.Area]);
area_cutoff = Area - 2.*std_nuclear_area;
del_nucleus_recom = find([S_nuclei.Area] < area_cutoff);
if ~isempty(del_nucleus_recom)
    fprintf('Recommended action: delete nucleus %i \n',del_nucleus_recom)
elseif area_cutoff < 0
    fprintf('Area cutoff is negative! Examine cc_nuclei for very small nuclei and delete them manually. \n')
else
    fprintf('Recommended action: keep all nuclei \n')
end

if area_cutoff < 0
fprintf('Type "dbcont" to continue, "dbquit" to exit! \n') 
keyboard
else
end

%%
% From the profile of each nucleus we calculate the global distance
% profile for all the cells in the image.

nuclei_list = [1:n_nuclei];

prompt = 'Accept recommendation? ''Y''/''N'' [Y] ';
str = input(prompt);
if isempty(str)||strcmp(str,'Y')||strcmp(str,'y')
    if isempty(del_nucleus_recom)
        del_nucleus = 0;
    else
        del_nucleus = del_nucleus_recom;
    end
else
    if ~exist('del_nucleus','var') || isempty(del_nucleus_recom) || isempty(del_nucleus)
        prompt = 'Enter nuclei to delete from mean (as vector; 0 for no deletion):  ';
        del_nucleus = input(prompt);
    end
end

nuclei_list = setdiff(nuclei_list,del_nucleus);

% Number of nuclei deleted from mean.
n_del_nucleus = n_nuclei - length(nuclei_list);

Radial_Profiles_mat = zeros(r+1,n_nuclei);
for i = 1:n_nuclei
Radial_Profiles_mat(:,i) = Radial_Profiles{i}(:,1);
end

Radial_Profiles_mean = mean(Radial_Profiles_mat(:,nuclei_list),2);
Radial_Profiles_std = std(Radial_Profiles_mat(:,nuclei_list),0,2);

Radial_profile_plus_bound = Radial_Profiles_mean + Radial_Profiles_std;
Radial_profile_minus_bound = Radial_Profiles_mean - Radial_Profiles_std;

[~,Radial_profile_plus_bound_peak_ind] = ...
    findpeaks(smooth(Radial_profile_plus_bound(1:dist_cutoff_ind,1)));
[~,Radial_profile_minus_bound_peak_ind] = ...
    findpeaks(smooth(Radial_profile_minus_bound(1:dist_cutoff_ind,1)));

if ~isempty(Radial_profile_plus_bound_peak_ind)
    Radial_profile_plus_bound_peak = dist_unique(Radial_profile_plus_bound_peak_ind(1))*pix2micr;
else
    Radial_profile_plus_bound_peak = NaN;
end

if ~isempty(Radial_profile_minus_bound_peak_ind)    
    Radial_profile_minus_bound_peak = dist_unique(Radial_profile_minus_bound_peak_ind(1))*pix2micr;
else
    Radial_profile_minus_bound_peak = NaN;
end

%%
Other_Feature_Global_Profile_plot_1 = figure;
% set(gcf,'Position', [0. 100 1120 840])

p1 = plot(dist_unique*pix2micr,Radial_Profiles_mean, 'b-', 'LineWidth', 2);hold on
% p2 = plot(dist_unique*pix2micr,Radial_profile_plus_bound, 'r--', 'LineWidth', 1);hold on
% p3 = plot(dist_unique*pix2micr,Radial_profile_minus_bound, 'r--', 'LineWidth', 1);hold on

x = dist_unique'*pix2micr;
% p4 = patch([x fliplr(x)], [Radial_profile_plus_bound' fliplr(Radial_profile_minus_bound')],[1,215/255,0],'FaceAlpha',0.15,'EdgeAlpha',0.);
p4 = patch([x fliplr(x)], [Radial_profile_plus_bound' fliplr(Radial_profile_minus_bound')],[1,1,0],'FaceAlpha',0.2,'EdgeAlpha',0.);

mnr = Radius*pix2micr;
ax = gca;
p5 = line([mnr mnr],ax.YLim,'Color','green','LineStyle','-','LineWidth', 2);
hold off

grid on;
legend([p1 p4 p5 ],{'Nuclei-OF dist. radial profile','+/- standard deviation','Mean nuclei radius'})
% title('Other feature-Nuclei distance distribution', 'FontSize', 12)
xlabel('Distance [\mu]')
ylabel('Vector density')
xlim([0 100])
% ylim([-5 25])
 
% disp('Press any key to continue !') 
% pause;

%% NUCLEUS to NUCLEUS DISTANCES 

% Here we use the radial profiles calculated previously.

% Mean and std of the peak positions
mnr_ind = uint64((mnr/pix2micr)+1);
dist_cutoff_ind_2 = uint64((mean_centr_dist/pix2micr)+1);
dist_cutoff_2 = dist_unique(dist_cutoff_ind_2)*pix2micr;
mdist_n = zeros(n_nuclei,1);
mdens_n = zeros(n_nuclei,1);
for i = 1:n_nuclei
    % [pd_n,pd_ind_n] = findpeaks(smooth(Radial_Profiles_n{i}(1:dist_cutoff_ind_2,1),5));
    [pd_n,pd_ind_n] = findpeaks(smooth(Radial_Profiles_n{i}(1:dist_cutoff_ind_2,1),'rlowess'),'SortStr','descend','NPeaks',3);

    if ~isempty(pd_n)
        mdens_n(i) = pd_n(1);
        mdist_n(i) = dist_unique(pd_ind_n(1))*pix2micr;
    else
        mdens_n(i) = NaN;
        mdist_n(i) = NaN;
    end
end
mean_mdist_n = nanmean(mdist_n);
std_mdist_n = nanstd(mdist_n);
mean_mdens_n = nanmean(mdens_n);
std_mdens_n = nanstd(mdens_n);
 
% The Patterson map is centrosymmetric. The center of the image is the
% F(0,0) vector (the vector of every pixel with itself). Since this is
% properly a "Difference Patterson Map" (1 - 2+3, as above) that point has
% 0 density. Hot regions are regions with high density of vectors. Next we
% calculate a radially averaged density for each Patteron map, which gives
% us the vector distribution.

if n_nuclei <= 100
    Nucl_to_nucl_Profiles_plot = figure;
    set(gcf,'Unit','Normalized','Position',[0 0.1 0.6 0.9]);
    for i = 1:n_nuclei
        subplot(6,ceil(n_nuclei/6),i)
        plot(dist_unique*pix2micr,Radial_Profiles_n{i}(:,1), 'b-', 'LineWidth', 2);
        hold on
        ax = gca;
        line([mnr mnr],ax.YLim,'Color','green','LineStyle','-')
        line([dist_cutoff_2 dist_cutoff_2],ax.YLim,'Color','red','LineStyle','-')
        grid on;
        title(['N' num2str(i)], 'FontSize', 12)
    if n_nuclei<30
        xlabel('Centr. Dist. [\mu]')
    end
        % ylabel('Vector density')
        xlim([0 100])
        xticks([0 20 40 60 80 100]);
        xticklabels([]);
    end
    fprintf('Nuclei already selected for deletion will not be added to the global profile \n')
else
    fprintf('Too many nuclei to display individual profiles. \n')
    fprintf('Nuclei already selected for deletion will not be added to the global profile \n')    
end

% From the profile of each nucleus we calculate the global distance
% profile for all the cells in the image.

Radial_Profiles_n_mat = zeros(r+1,n_nuclei);
for i = 1:n_nuclei
Radial_Profiles_n_mat(:,i) = Radial_Profiles_n{i}(:,1);
end

Radial_Profiles_n_mean = mean(Radial_Profiles_n_mat(:,nuclei_list),2);
Radial_Profiles_n_std = std(Radial_Profiles_n_mat(:,nuclei_list),0,2);

Radial_profile_n_plus_bound = Radial_Profiles_n_mean + Radial_Profiles_n_std;
Radial_profile_n_minus_bound = Radial_Profiles_n_mean - Radial_Profiles_n_std;

[nu_to_nu_peak_dens,nu_to_nu_peak_ind] = findpeaks(smooth((Radial_Profiles_n_mean(1:dist_cutoff_ind_2)),'rlowess'),'SortStr','descend','NPeaks',3);
nu_to_nu_peak_dist = dist_unique(nu_to_nu_peak_ind(1))*pix2micr;

%% Here we plot the nuclei-nuclei distance on top of the nuclei-OF distance

figure(Other_Feature_Global_Profile_plot_1);
% set(gcf,'Position', [0. 100 1120 840])
hold on

% Nucleus_to_nucleus_Global_Profile_plot_1 = figure;

p6 = plot(dist_unique*pix2micr,Radial_Profiles_n_mean, 'm-', 'LineWidth', 2);hold on

mnr = Radius*pix2micr;
ax = gca;
xlim([0 mean_centr_dist])
ylim(ax.YLim)
% ylim([-5 35])
% 
% p7 = plot(dist_unique*pix2micr,Radial_profile_n_plus_bound, 'r:', 'LineWidth', 1);hold on
% p8 = plot(dist_unique*pix2micr,Radial_profile_n_minus_bound, 'r:', 'LineWidth', 1);hold on
% mnr = Radius*pix2micr;

x = dist_unique'*pix2micr;
p9 = patch([x fliplr(x)], [Radial_profile_n_plus_bound' fliplr(Radial_profile_n_minus_bound')], 'b','FaceAlpha',0.06,'EdgeAlpha',0.);
p10 = line([mnr mnr],ax.YLim,'Color','green','LineStyle','-','LineWidth', 2);

legend1 = legend([p1 p4 p6 p9 p10 ],...
    {'Nuclei-myosin dist. radial profile','+/- standard deviation',...
    'Nuclei-nuclei dist. radial profile','+/- standard deviation','Mean nuclei radius'});
% set(legend1,'LineWidth',2,'FontSize',24);
% set(gca,'FontSize',38,'LineWidth',3);
% set(p5,'LineWidth', 3);
% set(p1,'LineWidth', 4);
% set(p6,'LineWidth', 4);
% ylabel('Vector density','LineWidth',2,'FontSize',24);
% xlabel('Distance [\mu]','LineWidth',2,'FontSize',24);

%% 
fprintf('Confirm mean Other feature-Nuclei distance with a single FFT on all nuclei? \n')
fprintf('Type "dbcont" to continue, "dbquit" to exit! \n') 
keyboard

%% MEAN DISTANCES (other feature-all nuclei)
% Here we do the analysis for all nuclei at once.

% Make masked copies of B and the other feature.
B_masked = B;
OF_masked = OF;

% Match nucleus deletion from list if del_nucleus ~= 0
if sum(del_nucleus) > 0
    for idel = 1:length(del_nucleus)
        del_nucleus_map = false(size(bw_nuclei));
        del_nucleus_map(cc_nuclei.PixelIdxList{del_nucleus(idel)}) = true;
        B_masked(del_nucleus_map) = 0;
    end
end

% 2D-FFT's
B_masked(~bw_nuclei) = 0;
OF_masked(~bw_other) = 0;

F_n = fft2(B_masked,m,n);
F_m = fft2(OF_masked,m,n);

F_nm = F_n + F_m;

% We carry out the subtraction in frequency space and do a single iFFT
p_d1 = abs(ifft2(real(F_nm).^2 - real(F_n).^2 - real(F_m).^2,'symmetric'));

% Here we show the Patterson maps after centering
p_d1 = fftshift(p_d1);

Patterson = figure;
% pat_axis = [-dist_mat(1025,1:1024)+1024 dist_mat(1025,1025:end)+1024];
% pat_axis_lab = [-dist_mat(1025,1:1024) dist_mat(1025,1025) dist_mat(1025,1026:end)];
pat_x_axis = [-dist_mat(m/2+1,1:n/2)+n/2 dist_mat(m/2+1,n/2+1:end)+n/2];
pat_x_axis_lab = [-dist_mat(m/2+1,1:n/2) dist_mat(m/2+1,n/2+1) dist_mat(m/2+1,n/2+2:end)];
pat_y_axis = [-dist_mat(1:m/2,n/2+1)+m/2 ; dist_mat(m/2+1:end,n/2+1)+m/2];
pat_y_axis_lab = [-dist_mat(1:m/2,n/2+1) ; dist_mat(m/2+1,n/2+1) ; dist_mat(m/2+2:end,n/2+1)];

set(gcf,'Unit','Normalized','Position',[0 0.3 0.45 0.7])
% subplot(1,2,1);
prime_m = factor(m);
prime_n = factor(n);
imagesc(p_d1);colormap(jet);
skip_m = prod(prime_m(5:end));
skip_n = prod(prime_n(5:end));
x_ticks = pat_x_axis(1:skip_m:end);
for i = 1:length(x_ticks)
    x_tick_ind = x_ticks(i) + 1;
    x_tick_label{i} = num2str(round(pat_x_axis_lab(x_tick_ind)*pix2micr));
end

y_ticks = pat_y_axis(1:skip_n:end);
for i = 1:length(y_ticks)
    y_tick_ind = y_ticks(i) + 1;
    y_tick_label{i} = num2str(round(pat_y_axis_lab(y_tick_ind)*pix2micr));
end

xticks(x_ticks);
xticklabels(x_tick_label);
xtickangle(90);
yticks(y_ticks);
yticklabels(y_tick_label)
xlabel('Dist. from nucleus centroid [\mu]')
ylabel('Dist. from nucleus centroid [\mu]')
grid on
% subplot(1,2,2);
% imagesc(dist_mat);colormap(jet); colorbar

% Radial profiles
radial_profile_p_d1 = zeros(length(dist_unique),1);

for j = 1:nunique
    radial_profile_p_d1(j) = mean(p_d1(k{j}));
end

% Scale the radial profile by the sum of all nuclei areas*intensities
radial_profile_p_d1 = radial_profile_p_d1/ ...
    sum([S_nuclei(nuclei_list).Area] .* [S_nuclei(nuclei_list).MeanIntensity]);

% Mean other feature distance from the nucleus centroid
Other_Feature_Global_Profile_plot_2 = figure;
plot(dist_unique*pix2micr,radial_profile_p_d1, 'b.', 'LineWidth', 2);hold on
ax = gca;
line([mnr mnr],ax.YLim,'Color','green','LineStyle','-')
xlim([0 100]);
grid on;
legend('Mean dist. other feature:nuclei centroids', 'Mean nuclei radius')
title('Other feature-Nuclei distance distribution', 'FontSize', 12)
xlabel('Distance from nucleus centroid [\mu]')
ylabel('Vector density')

% Mean other feature density peak
[mean_dens_0,m_peak_0] = max(Radial_Profiles_mean(1:dist_cutoff_ind));
[mean_dens_1,m_peak_1] = max(radial_profile_p_d1(1:dist_cutoff_ind));
mean_peak_0 = dist_unique(m_peak_0)*pix2micr;
mean_peak_1 = dist_unique(m_peak_1)*pix2micr;

% Peak density without radial average. 
[~,max_peak_1_ind] = max(p_d1(:));
max_peak_1 = dist_mat(max_peak_1_ind)*pix2micr;

%%
fprintf('Distance calculation is complete! \n')
fprintf('Calculate summary tables? \n')
fprintf('If so, Type "dbcont" to continue, "dbquit" to exit! \n') 
keyboard

%% Summary Table
% Recall we have calculated the following:
% Maj_AL = mean([S_nuclei.MajorAxisLength])
% Min_AL = mean([S_nuclei.MinorAxisLength])
% Peri = mean([S_nuclei.Perimeter])
% Circ = mean([S_nuclei.Circularity])
% Eccentr = mean([S_nuclei.Eccentricity])
% Area = mean([S_nuclei.Area])
% Conv_area = mean([S_nuclei.ConvexArea])
% Centroids = reshape([S_nuclei.Centroid],2,18)'
% AL_ratio = Maj_AL/Min_AL
% Radius = mean([S_nuclei.EquivDiameter])/2
% Mean_int = [S_nuclei.MeanIntensity]
%     
% nucl_cell_ratio = sum(bw_nuclei(:))/sum(bw_cell(:))
% nucl_cyto_ratio = sum(bw_nuclei(:))/sum(bw_cyto(:))
% 
% mito_cyto_ratio = sum(bw_mitos(:))/sum(bw_cyto(:))
% mito_cell_ratio = sum(bw_mitos(:))/sum(bw_cell(:))
% mito_nuclei_ratio = sum(bw_mitos(:))/sum(bw_nuclei(:))
n_nuclei_final = length(nuclei_list);
mean_cell_area = (sum(bw_cell(:))*pix2micr^2)/n_nuclei_final;
mean_cyto_area = mean_cell_area - Area*pix2micr^2; 
mean_other_feature_area = (sum(bw_other(:))*pix2micr^2)/n_nuclei_final;

Nuclei_mat = [[S_nuclei.Area]'*pix2micr^2 [S_nuclei.Eccentricity]' ...
               [S_nuclei.MajorAxisLength]'*pix2micr [S_nuclei.MinorAxisLength]'*pix2micr ... 
               [S_nuclei.MajorAxisLength]'./[S_nuclei.MinorAxisLength]' ...
               mean_centr_dist_by_nucl mdist_n mdens mdist ...
               [S_nuclei.Area]'*pix2micr^2/mean_cell_area ...
               [S_nuclei.Area]'*pix2micr^2/mean_cyto_area ...
               [S_nuclei.Area]'*pix2micr^2/mean_other_feature_area];
Nuclei_mat = Nuclei_mat(nuclei_list,:);
Variables_nuclei = {'nucl_area' 'nucl_eccentr' 'Maj_AL' 'Min_AL' 'AL_ratio' ...
                    'mean_centr_dist_direct' 'mean_centr_dist_fft' ...
                    'peak_other_feature_dens' 'peak_other_feature_dist' ...
                    'nucl_cell_ratio' 'nucl_cyto_ratio' 'nucl_other_feature_ratio'};
T1 = array2table(Nuclei_mat,'VariableNames',Variables_nuclei)

del_nucleus_list = zeros(1,3);
for idel = 1:length(del_nucleus)
    del_nucleus_list(idel) = del_nucleus(idel);
end

Summary_cell = {n_nuclei_final Radius*pix2micr Eccentr Area*pix2micr^2 ...
                (sum(bw_nuclei_uncut(:))*pix2micr^2)/n_nuclei ...
                Maj_AL*pix2micr Min_AL*pix2micr AL_ratio ...
                mean_centr_dist nu_to_nu_peak_dist ...
                (sum(bw_cell(:))*pix2micr^2)/n_nuclei ...
                (sum(bw_other(:))*pix2micr^2)/n_nuclei uncut_nucl_cell_ratio ...
                uncut_nucl_cyto_ratio nucl_cell_ratio ...
                nucl_cyto_ratio other_cell_ratio other_cyto_ratio ... 
                other_nuclei_ratio max_peak_1 ...
                mean_peak_1 mean_dens_1};
            
Variables_all = {'no_of_nuclei' 'mean_nucl_radius' 'nucl_eccen' 'mean_nucl_area' ...
    'mean_uncut_nucl_area' 'Maj_AL' 'Min_AL' 'AL_ratio' ...
    'mean_centr_dist' 'neighbour_centr_dist' 'mean_cell_area' ...
    'mean_other_feature_area' 'uncut_nucl_cell_ratio' 'uncut_nucl_other_feature_ratio' ...
    'nucl_cell_ratio' 'nucl_other_feature_ratio' ...
    'other_feature_cell_ratio' ... 
    'other_feature_cyto_ratio' 'other_feature_nuclei_ratio' ...
    'peak_other_feature_dist' 'mean_other_feature_dist' ...
    'mean_other_feature_dens' };

T2 = cell2table(Summary_cell,'VariableNames',Variables_all)

%%
fprintf('All data processing is completed. If you are not satisfied this is the moment to stop ! \n')
prompt = 'Do you want to quit? ''Y''/''N'' [use quotes] [N]: ';
str = input(prompt);
    if isempty(str)||strcmp(str,'N')||strcmp(str,'n')
    else
        return    
    end

fprintf('Are you sure you want to save the tables to text files? \n')
fprintf('Type "dbcont" to continue, "dbquit" to exit! \n') 
keyboard

dlmwrite([header '/' header '_Images_Nuclei_Features_Low_Res.txt'],Nuclei_mat,'-append',...
'delimiter','\t','roffset',1)

A = zeros(2,length(Summary_cell));
A(1,1) = img_no;
A(2,:) = cell2mat(Summary_cell);
dlmwrite([header '/' header '_Images_All_Properties_Low_Res.txt'],A,'-append',...
'delimiter','\t','roffset',1)

%%
fprintf('Close all figures and save results to a .mat file? \n') 
fprintf('Type "dbcont" to continue, "dbquit" to exit! \n') 
keyboard

close all
save([header '/' header_2 '_Composite_RGB.mat'])

% end
